! Theophanis V. Karambas
! Diffraction at a semi-infinite breakwater
! Analytical solution of the Helmholtz equation
! by Penney and Price (1944)

	complex i1, akdc,aI1,aI2
	real (8) lm1,lm2,s1,c1,s2,c2


	write (*,*) 'r (in m)   thita (in deg)     bita (in deg)  ?'
	read(*,*) r,thita,bita

	write (*,*) 'period (in sec)    depth  (in m)   ?'
	read (*,*) per,depth


      pi=4.0*atan(1.0)

	thita=(thita/90.)*pi/2.
	bita=(bita/90.)*pi/2.
	
	i1=(0,1)

	ALo=9.81*(PER**2)/(2*PI)
	ALNGHP=ALo

! estimation of L from L=Lo tanh(kd)
	SIGMA=2*PI/PER
	PSI=(SIGMA**2)*depth/9.81
	HLHUNT=1+.6522*PSI+.4622*(PSI**2)+.0864*(PSI**4)+.0675*PSI**5
	HUNT=1.0/(PSI+1.0/HLHUNT)
	ALNGH=PER*SQRT(9.81*depth)*SQRT(HUNT)
39    AKAPA=2*PI/ALNGHP
	AKD=AKAPA*depth
	ALNGH=ALo*TANH(AKD)
	DLEN=ABS(ALNGH-ALNGHP)
	IF (DLEN.GT.0.001) THEN
	ALNGHP=ALNGH
	GOTO 39
	ELSE
	END IF

	aL=ALNGH
	ak=2.*pi/aL


	lm1=-sqrt(4.*ak*r/pi)*sin((thita-bita)/2.)
	lm2=-sqrt(4.*ak*r/pi)*sin((thita+bita)/2.)

	call frenel (lm1,s1,c1)
	call frenel (lm2,s2,c2)


	aI1=0.5*(1+s1+c1)+0.5*i1*(c1-s1)
	aI2=0.5*(1+s2+c2)+0.5*i1*(c2-s2)


	akdc=aI1*exp(-i1*ak*r*cos(thita-bita))+
     &	aI2*exp(-i1*ak*r*cos(thita+bita))

	write(*,*)
	write(*,*)
	write(*,*)
	write(*,*)
	write(*,*) ' Diffraction coefficient:'
      write(*,126) abs(akdc)
	write(*,*)
	write(*,*)
	write(*,*)	
	write(*,*)
126   format(f32.4)

	contains
      SUBROUTINE frenel(x,s,c)
! Fresnel integrals
      INTEGER MAXIT
      REAL (8) c,s,x,EPS,FPMIN,PI,PIBY2,XMIN
      PARAMETER (EPS=6.e-8,MAXIT=100,FPMIN=1.e-30,XMIN=1.5,PI=3.1415927
     & ,PIBY2=1.5707963)
      INTEGER k,n
      REAL a,absc,ax,fact,pix2,sign,sum,sumc,sums,term,test
      COMPLEX b,cc,d,h,del,cs
      LOGICAL odd
      absc(h)=abs(real(h))+abs(aimag(h))
      ax=abs(x)
      if(ax.lt.sqrt(FPMIN))then
        s=0.
        c=ax
      else if(ax.le.XMIN)then
        sum=0.
        sums=0.
        sumc=ax

        sign=1.
        fact=PIBY2*ax*ax
        odd=.true.
        term=ax
        n=3
        do 11 k=1,MAXIT
          term=term*fact/k
          sum=sum+sign*term/n
          test=abs(sum)*EPS
          if(odd)then
            sign=-sign
            sums=sum
            sum=sumc
          else
            sumc=sum
            sum=sums
          endif
          if(term.lt.test)goto 1
          odd=.not.odd
          n=n+2
11      continue
        pause 'series failed in frenel'
1       s=sums

        c=sumc
      else
        pix2=PI*ax*ax
        b=cmplx(1.,-pix2)
        cc=1./FPMIN
        d=1./b
        h=d
        n=-1
        do 12 k=2,MAXIT
          n=n+2
          a=-n*(n+1)
          b=b+4.
          d=1./(a*d+b)
          cc=b+a/cc
          del=cc*d
          h=h*del
          if(absc(del-1.).lt.EPS)goto 2
12      continue
        pause 'cf failed in frenel'
2       h=h*cmplx(ax,-ax)
        cs=cmplx(.5,.5)*(1.-cmplx(cos(.5*pix2),sin(.5*pix2))*h)
        c=real(cs)

        s=aimag(cs)
      endif
      if(x.lt.0.)then
        c=-c
        s=-s
      endif
      
       END subroutine frenel

	end
